housing = ['北京路19号新华苑(北京路)', '盐务街1号盐务街1号院', '盐务街19号盐务街19号院', '贵乌中路与幼育巷交叉口西240米云岩区人才公寓', '中华北路与北京路辅路交叉口东120米贵医附院平安小区', '贵乌社区服务中心贵乌南路华燕楼', '贵乌南路108号冠竹苑', '友谊路阿一鲍鱼旁锦绣苑', '贵乌南路178号阳光都市商住小区', '兴贵路21号兴贵路21号院', '北京路66号(北京路地铁站D口步行440米)北京路66号院', '贵乌南路156号凯发家园', '盐务街86号盐务街86号院', '兴贵路23号兴贵路23号院', '宝山北路凯福苑', '幼育巷与贵乌中路交叉口西北120米麒龙润园小区', '百草巷25号百草巷25号院', '幼育巷1号幼育巷-1号院', '普陀路156号(友谊路与贵乌南路交汇)变维小区', '友谊路77-89号友谊住宅', '友谊路198号百灵阳光小区', '白鹦巷与普陀路交叉口北140米中环社区白鹦巷1号院落', '普陀社区普陀路90号院', '余家巷34号余家巷34号院', '盐务街88号盐务街88号院', '兴贵路26号兴贵路24号院', '余家巷19号余家巷19号院', '普陀社区服务中心宝山北路368号普陀社区', '友谊路258号百灵·尚品一号', '金仓路与吉祥路交叉口南160米兴贵路二六号院', '普陀路与陕西路交叉口东120米余家巷小区', '盐务街102号烟草局小区', '市北路26号市北路26号院', '普陀路1号千禧苑', '余家巷15号余家巷15号院', '狮子路4号相宝居委会小区', '市北路38号市北路38号院', '新添大道南段5号东安新苑', '普陀社区服务中心普陀路天主教堂对面黔达花园', '中华北路157号1仁达饭店小区', '余家巷12号余家巷12号院', '普陀社区服务中心宝山路贵州日报社旁施格名门', '中华社区服务中心中华北路79号附1号毛光翔公馆', '盐务街宅吉小区吉祥小区', '新添大道南段85号博泰华庭', '陕西路141号陕西路141号院', '大营路与新添大道南段辅路交叉口东100米亚华小区', '余家巷8号余家巷8号院', '市北路15号市北路15号院', '普陀路与白鹦巷交叉口西南40米华城凯旋门小区', '盐务街金仓路60号宅学', '宝山北路辅路与新印厂路交叉口东北100米新华印刷厂四合院小区', '延中社区服务中心公园北路飞山街新型社区温馨家园', '盐务街宅吉小区', '和平路虎门巷9号华谊苑', '盐务街63号中天宅吉碧苑', '北京路104号(北京路地铁站C口步行480米)银杏小区', '新添大道南段93号新添大道南段93号院', '陕西路127号陕西路127号院', '新添大道南段62号贵开花园', '圣发道与亿达路交叉口东北180米人才公寓', '亿达路2号青年公寓1期', '海关巡视路与远航路交叉口东南60米中国铁建大桥局空客项目部生活区', '茗华道与贵昌路交叉口西北220米枫愉园', '贵昌路与隆华道交叉口南180米枫悦园', '金桥街道贵昌路与景怡道交叉口西北160米军粮城2期', '军粮城2期枫舒园', '军粮城2期枫畅园', '中环南路76号天保青年公寓', '军粮城二期贵昌路与北旺道交口怡心里', '军粮城2期物业景文轩', '贵雅路与景文道交叉口西北220米怡贤里', '景怡道与贵达路交叉口西北180米景云轩', '新立街军粮城二期怡盛里', '军粮城二期东羽小学对面景晴轩', '么六桥龙祥里', '金桥街龙泉里', '东丽区开发区龙城里', '民航大学以东约3.2公里(东丽民族中学以南)瑜芳园', '景雅道与长盛路交叉口西220米仁欣家园', '津北公路与万明路交叉口西180米龙港里', '津塘公路仁乐家园', '津北公路与外环线交口港城温泉花园', '津塘公路7号仁静家园', '中环西路60号致馨公寓', '军粮城街道军粮城新市镇富贵路与景怡道交口中街商业广场尚河城', '景怡道与富锦路交叉口西160米冬梅轩', '中环东路福光职工公寓(西门)附近福光公寓', '津塘公路北民族路东65号蓝领公寓', '津塘公路七号桥小东庄南侧(小东庄地铁站A口步行80米)悦盛园', '东丽津塘公路小东庄站(小东庄地铁站A口步行330米)小东庄东盛园', '空港物流加工区西七道1号凤凰墅', '军粮城大街辅路与塘洼道交叉口西南220米军粮城1期', '东七道与航海路交叉口西100米欣欣家园', '空港国际物流区空港经济区中心大道188号复地温莎堡', 'S51宁静高速辅路与先锋东路交叉口东360米小东庄1区', '中心大道与东六道交口远航商务中心', '东丽区开发区三经路蓝庭公寓(合兴路)', '保税区空港国际物流区空港经济区西七道复地·温莎堡-红坊区', '中心大道与东七道交汇处远洋新干线', '东丽开发区津塘路与五经路交口处帝达·东谷国际', '东七道49号云栖苑', '东丽区开发区东丽开发区(宜家家居)后面蓝庭中心', '旌智道与新英路交叉口西260米万科天地', '东七道北侧津悦华庭', '津塘公路轻轨开发区站北侧万科魅力之城2期', '保税区空港国际物流区西五道高尔夫温泉别墅', '军粮城街道东金路军悦里', '空港物流加工区规划路与东五道交口万科新里程花园', '津塘公路全境空港自贸区万科新里程家园', '津塘公路万科魅力之城', '津塘公路轻轨开发区站北侧(外环外2.5公里)万科魅力之城1期', '东五道与中心大道交叉口东北100米望海阁', '军粮城镇永兴村南大街南250米军粮城和顺家园一区', '和顺家园四区(中南二街北300米)军粮城和顺家园四区', '新桂路与运智路交口处东南侧蓝庭国际', '丽北路四合新城', '军粮城街道军粮城铁路地道口北军星辅城', '中心大道与东五道交口颐景公寓', '春晓路与中环西路交叉口东北160米万顺高尔夫行政公寓', '南阳街道雷山小区向阳路向阳嘉苑', '南虹路与南庄路交叉口西南220米向阳嘉苑雷山东风', '左十四线附近向阳嘉苑雷山远大', '南临路与南庄路交叉口东南120米成轩苑', '潮都北路1250号(南阳地铁站C口步行450米)潮上云兴雅苑', '向阳路766号潮竞云望城(一区)', '潮都西路107号潮上云临南苑', '向阳路338正北方向120米南翔名苑', '南临路与南阳大道交叉口东北180米潮上云临北苑', '南阳街道红山家苑', '南阳街道办事处向西300米潮都华庭', '港城大道与南丰路交叉口(南阳地铁站B口步行340米)大诚纪名府', '潮都西路与港城大道交叉口西南140米阅杭里', '港城大道与南阳大道交叉口北100米德信浙旅天空之城', '滨江二路与港城大道交叉口东北20米会展新城', '南阳镇赭山永里路赭东家苑', '南丰路与岩南线交叉口东南120米南丰东苑', '南丰新村西区(岩南线南100米)南丰西苑', '南翔路与南临路交叉口西南160米南兴西苑', '永里路与永美线交叉口东360米赭东小区', '惠南街129-1号(向阳路地铁站D口步行340米)南洋小区', '南虹路与美狮路交叉口东北320米聆空云境城', '南虹路450号附近南兴东苑', '惠南街与宝丰路交叉口西40米南阳街道南阳社区', '南虹路380号江南公寓', '南阳街道南阳经济开发区天翔赭山家园', '阳城路龙虎新村', '南阳镇赭山坞里村圣山公寓', '南虹路528号龙虎村新区', '南虹路与横蓬路交叉口西80米家和城', '岩峰村安置区(左十四线西300米)岩峰村安置区', '岩南线与美狮路交叉口北380米别院潮苑', '观十五线旁潮闻天下', '河庄南路1629号杭州德乾实业有限公司公寓楼', '河庄路与港城大道交叉口东北500米蓬园村第一安置小区', '左十四线与永丰路交叉口东南440米沈氏雅居', '钟秀街与河庄路交叉口西360米建设村小区', '三联村清风公寓', '河庄街道河景路1号浙江军创园', '永丰路819号贝壳新青年公寓(河庄店)', '河庄街道河庄路(河庄路地铁站D口步行320米)御东湾', '靖南公寓', '经三路同二村新区', '河庄镇永丰路(镇政府对面)宏河嘉园', '同三线与永丰路交叉口(河庄路地铁站D口步行90米)天宇景悦家苑', '河庄街道河景路(河庄派出所对面)金诺加州洋房', '纬十一路与河中路交叉口东南140米同一小区配套房', '河景路与河庄路交叉口东40米江城河庄', '采慧巷与永丰路交叉口东北100米帝景园南区',
           '邮电路20号金诺房产小区', '永丰路与河中路交叉口帝景园北区', '钱塘区府东路河庄形象城北侧大江之星', '靖江街道育才路西萧山图书馆黎明社区', '北锋路与先锋路交叉口东北320米同德社区', '河景路与潮启巷交叉口东南80米大江之星东区', '河庄大道1111号(河庄路地铁站B口步行250米)金色和庄', '商贸街与靖江路交叉口西北20米商贸街慢生活街区', '向阳路与青六线交叉口西南角星悦湾', '建新线与塘新线交叉口西北20米新和村', '靖中路与朱家弄交叉口东20米安澜桥村', '香港世界地质公园西贡火山岩园区', '滘西村故事馆', '清水湾海宁居', '柏涛径柏涛湾', '颖翠小筑(东南门)附近相思湾20号', '龙虾湾路与茅莆路交叉口南220米弥勒殿', '坑口永隆路33号愚园', '新界清水湾坑口路海景别墅', '永隆街26号海岚居', '相思湾路碧云苑', '清水湾Hang Hau Wing Lung Road Dd.238 Lot 533棕林别墅', '香港五块田棕林苑', '大环头路清水湾花园', '清水湾下洋路帝涛居', '早禾路18号早禾居', '下洋新村路下洋新村', '碧沙路19号海逸居', '大纲仔路东50米早禾美居', '清水湾陈屋村', '碧沙路10号满湖花园', '银岬路15号海滨别墅', '银岬路29号金湖别墅', '银岬路25号海湾别墅', '银岬路11号富嘉堡', '银岬路18号夏威夷花园A5-A6', '银岬路23号金碧苑Ⅱ', '清水湾道胜景别墅', '9号/NO,翠湖别墅9 Silver Cape Rd,Sai Kung翠湖别墅', '西沙路与大网仔路交叉口东340米威都别墅', '银泉径南50米海天湾', '银岬路26号金碧苑第1期', '西贡大网仔路榕园别墅', '银线湾道5号银海山庄', '银岬路5号爱琴居', '清水湾银台路15号碧涛花园', '清水湾道850号颍川', '银星径9号湾景台', '银峦路12号Bougain Villa Gardens', '安宁径1号金碧小筑', '银峦路7号华富花园', '清水湾道328号清雅苑', '银湖路1号彩涛别墅', '康健路泰湖阁', '银线湾银峦路18号龙湖别墅', '康健路288号西贡涛苑', '香港坑口培成里8号明德村', '康健路与华容路交叉口西北80米渔民村', '将军澳昭信路6号显明苑', '将军澳坑口宝宁路25号富宁花园', '常宁路将军澳第39/41区德裕楼', '西贡福民路高富楼C座', '香港坑口培成路38号,Tseung Kwan O培成花园', '福民路与惠民路交叉口东南40米高富楼', '香港坑口昭信路10号炜明阁', '輋径笃路与蕉坑交叉口东南480米蕉坑口', '坑口银澳路2号煜明苑', '将军澳培成里2号乐颐居', '坑口常宁路1号裕明苑', '大网仔路706地段爱月楼', '坑口银澳路8号和明苑A座和逸阁和逸阁', '东升街4号附5号明月沱小区', '明月沱东升街35号阳光新苑', '004乡道南100米银月湖畔之愚乐坊', '广阳镇塘坎村河口场东港家园', 'S103(南涪路)重庆国际生物城', '鱼嘴镇人民政府棠富园', '和顺路与和顺一支路交叉口南180米棠锦园', '两江大道与和顺路交叉口东北180米瑞祥家园', '麻柳大道与天池大道交叉口东北320米生物城人才公寓', '鱼嘴西路兰亭花园', '郭益路郭家沱安置房', '和顺一支路与和煦路交叉口东南140米巨龙江山国际三期', '和煦路10号巨龙·江山国际四期', '和煦路107号和煦家园', '和煦路30号巨龙·江山国际', '郭益路和悦家园', '和煦路10号巨龙·江山国际二期', '鱼嘴北路与鱼嘴西路交叉口(鱼嘴地铁站2A口步行210米)金鑫花园', '和煦路30号(鱼嘴地铁站4A口步行420米)巨龙·江山国际一期', '鱼嘴镇(鱼嘴地铁站2A口步行220米)和韵家园南区', '鱼嘴地铁站1号口步行110米三峡移民小区', '和韵路47(鱼嘴地铁站2A口步行380米)和韵家园', '永和路与和煦路交叉口西北160米中商·两江公馆', '和韵路48号(鱼嘴地铁站2A口步行460米)和韵家园北区', '木洞镇土地垴农民新村土地垴农民新村香山郡社区', '新能雅苑', '103乡道宝科鑫天地', '和顺路238号(雁坪地铁站1号口步行310米)和锦家园A区', '和顺路238号(雁坪地铁站1号口步行110米)和锦家园', '和顺路236号(雁坪地铁站2B口步行80米)和锦家园B区', '鱼嘴镇人民政府(雁坪地铁站2B口步行70米)康韵家园', '松子村附近钱佳花园', '木洞镇小河边街78号新星节能小区', '康泰路69号重庆中农联合产业园A区', '长茂路29号两江汽车零部件物流园', '大石路与新建路交叉口东北80米丁家花园', '渝开大道美捷鱼复生产基地', '绕城渝宜互通与G50沪渝高速交叉口西北180米祥韵家园A区', '沪渝高速公路(石河清地铁站2号口步行310米)祥韵家园', '大桥一路与新建路交叉口东南280米港湾小区', '绕城渝宜互通东150米(石河清地铁站2号口步行200米)祥韵家园B区', '渝景大道191号清溪雅苑', '复龙路8号附近和乐家园', '渝景大道192号东原江山印月二期', '庆福路2号和庆家园A区', '庆福路2号东原江山印月', '安福路与渝景大道交叉口东160米三一云城', '山涧锦苑小区', '鱼嘴镇人民政府双玉路(双溪站西侧约100米)和美家园', '铁山坪药材社国防教育中心右恻60米铁山松香庭', '五宝镇马井村五一水库马岭·映月坊', '玉峰山村(铁山坪森林公园旁)东方山水', '鱼嘴镇双溪口村双溪福居B区', '坪玉峰山九里香堤', '潭溪路双溪雅苑A区', '铁山坪翠微路123号松香翠岭', '迎龙互通与G5001重庆绕城高速交叉口西100米迎龙创新港人才公寓', '黄明路北150米龙城大院', '迎龙镇镇政府对面北星小区', '翠微路与芳菲路交叉口富春山居', '胜利路171弄环龙苑', '胜利路169弄1-69号环庆新苑', '胜利路851弄胜利小区', '川沙路2015弄1-81号大湾名苑', '胜利路836弄张江东区人才公寓', '胜利路30弄1-65号金盛佳苑', '上海新雅东路125弄幸福里', '霞玉路436弄金海华城恒裕嘉园', '霞玉路80弄1-46号金海华城·桐和佳苑', '龚丰路576弄1-40号金海华城丰硕茗苑', '虹盛路250弄(王港镇上丰路)鑫港苑', '川沙路2373号光明·唐镇壹号', '霞玉路57弄金海华城韵秀佳苑', '川沙路2278弄华昌小区', '虹盛东路39弄1-34号边门金枫公寓', '龚新路龚新路549弄小区', '龚华路479弄龚路新城', '唐镇新雅路190号王港卫生院东10米新雅小区', '云雅路395弄1-80号阳光花城同馨苑', '虹昌路与虹盛路交叉口南60米虹盛小区东区', '龙东大道5401弄21-25号王桥小区', '云雅路555弄1-31号翔融苑', '龚路支路473弄金海华城康平佳苑', '宏雅路150弄保集·澜苑', '曹路镇龚华路银峰路恒越新都汇', '川沙路与茂丰路交叉口北40米银丰公寓', '银峰路549弄1-171号银丰苑', '龚华路50弄1-93号丰怡苑', '龚华路35弄(曹路地铁站3号口步行200米)金海华城永华苑', '庆南路与前哨路交叉口北140米前哨路198弄小区', '川沙新镇川沙路3321弄泰星公寓', '庆荣路379弄1-7号庆荣苑', '前哨路288弄前哨路288弄小区', '德淳路299弄浦发有家唐镇社区', '环庆中路57弄永旭家苑', '玉盘北路281弄浦发檀府', '金钻路1313弄金海华城华彩新苑', '庆荣路485弄1-31号荣华苑', '环庆中路323弄1-28号庆华花苑', '环庆中路168弄1-66号地纬家苑', '唐龙路1399号(玉盘北路288弄)仁恒东郊花园', '创新路21弄1-4号创新路21弄小区', '德淳路99弄紫金九号', '海顺路与金钻路交叉口东北220米金钻苑南区海顺路359弄', '庆滨路155弄5号海馨苑', '民雷路757弄1-337号万科第五园', '金晓路289弄1-33号星金家园', '德淳路100弄华侨城纯水岸', '海顺路359弄1-55号金钻苑南区', '海松路52弄1-52号星颂家园']



import ipaddress
from ipaddress import IPv4Address, IPv6Address
from time import sleep
from typing import Any, Generator


class NetLib:

    @staticmethod
    def cidr(address: str, netmask_or_prefix: int | str) -> dict[str, Any] | ValueError:
        """
        根据指定的网络和前缀长度计算CIDR信息。

        参数:
        - network: 字符串，指定的网络地址，可以是IPv4或IPv6格式。
        - prefix: 整数，网络的前缀长度。

        返回值:
        - 字典，包含CIDR信息，包括主机数量、地址范围、网络地址、广播地址和子网掩码。
        - 如果参数无效，返回ValueError异常。

        Examples:
        >>> print(Net.cidr('192.168.1.0', 24))

        >>> print(Net.cidr('240e::0', 64))
        """

        try:
            ip_type: ipaddress.IPv6Network | ipaddress.IPv4Network = ipaddress.IPv6Network if ':' in address else ipaddress.IPv4Network
            cidr = ip_type(f'{address}/{netmask_or_prefix}')

            # 输出网络地址、子网掩码、广播地址等信息
            host_count = cidr.num_addresses - 2
            range = str(cidr.network_address + 1) + '-' + \
                str(cidr.broadcast_address - 1)
            network_address = str(cidr.network_address)
            broadcast_address = str(cidr.broadcast_address)
            netmask = str(cidr.netmask)
            prefixlen = cidr.prefixlen

            return {'host_count': host_count, 'range': range, 'network_address': network_address, 'broadcast_address': broadcast_address, 'prefixlen': prefixlen, 'netmask': netmask}

        except ValueError as e:
            return e

    @staticmethod
    def ipaddress_generator(number: int, max: int | None = None, is_ipv6: bool = False,  **kwargs) -> Generator[IPv4Address, IPv6Address, None]:
        """
        生成指定数量的IP地址序列。

        Args:
        - number: 每次生成的IP地址数量。
        - max: 生成IP地址的最大数量, 为空时使用 number。
        - is_ipv6: 是否生成IPv6地址，默认生成IPv4地址。
        - **kwargs: 可选参数，用于指定IP地址的a、b、c、d、e、f、g、h初始值。

        Returns:
        - Generator: 生成器，每次返回number个IP地址组成的列表，直到生成max个IP地址为止。

        Examples:
        >>> for i in NetLib.ipaddress_generator(10, max=100):
        ...    print(";".join(map(lambda x: f"http://{str(x)}", i)))

        """
        a: int = kwargs.get("a", 1)
        b: int = kwargs.get("b", 1)
        c: int = kwargs.get("c", 1)
        d: int = kwargs.get("d", 1)
        e: int = kwargs.get("e", 1)
        f: int = kwargs.get("f", 1)
        g: int = kwargs.get("g", 1)
        h: int = kwargs.get("h", 1)

        if max is None:
            max = number

        if is_ipv6 is False:
            a_range: int = 256
            b_range: int = 256
            c_range: int = 256
            d_range: int = 255
            ip_list: list = []
            count: int = 0
            for _ in range(1, max+1):
                ip_list.append(ipaddress.IPv4Address(f"{a}.{b}.{c}.{d}"))
                count += 1
                d += 1

                if d == d_range:
                    d = 1
                    c = c+1
                    if c == c_range:
                        c = 1
                        b = b+1
                        if b == b_range:
                            b = 1
                            a = a+1
                            if a == a_range:
                                a = 1

                if len(ip_list) % number == 0:
                    yield ip_list
                    ip_list = []
                if max == count and len(ip_list) != 0:
                    yield ip_list
        else:
            a_range: int = 65536
            b_range: int = 65536
            c_range: int = 65536
            d_range: int = 65536
            e_range: int = 65536
            f_range: int = 65536
            g_range: int = 65536
            h_range: int = 65536
            ip_list: list = []
            count: int = 0
            for _ in range(1, max+1):
                ip_list.append(ipaddress.IPv6Address(
                    f"{a:04x}:{b:04x}:{c:04x}:{d:04x}:{e:04x}:{f:04x}:{g:04x}:{h:04x}"))
                count += 1
                h += 1

                if h == h_range:
                    h = 1
                    g = g+1
                    if g == g_range:
                        g = 1
                        f = f+1
                        if f == f_range:
                            f = 1
                            e = e+1
                            if e == e_range:
                                e = 1
                                d = d+1
                                if d == d_range:
                                    d = 1
                                    c = c+1
                                    if c == c_range:
                                        c = 1
                                        b = b+1
                                        if b == b_range:
                                            b = 1
                                            a = a+1
                                            if a == a_range:
                                                a = 1

                if len(ip_list) % number == 0:
                    yield ip_list
                    ip_list = []
                if max == count and len(ip_list) != 0:
                    yield ip_list

    @staticmethod
    def ip_in_subnet(ip_str, subnet_str):
        """

        Example:
        ip = '2a00::110:133'
        subnet = '2a00::110:0/120'

        if ip_in_subnet(ip, subnet):
            print(f"{ip} 在子网 {subnet} 内")
        else:
            print(f"{ip} 不在子网 {subnet} 内")
        """

        # 将 IP 地址和子网掩码字符串转换为 IPv4Network 对象
        ip_network = ipaddress.ip_network(subnet_str)
        # 将 IP 地址字符串转换为 IPv4Address 对象
        ip_address = ipaddress.ip_address(ip_str)
        # 判断 IP 地址是否在子网内
        return ip_address in ip_network

    @staticmethod
    def ipv6_in_range(ip_str, start_str, end_str):
        """
        Example:

        ip = '2a00::110:133'
        start = '2a00::110:13'
        end = '2a00::110:133'

        if ipv6_in_range(ip, start, end):
            print(f"{ip} 在范围 {start} - {end} 内")
        else:
            print(f"{ip} 不在范围 {start} - {end} 内")

        """

        # 将起始和结束地址字符串转换为 IPv6Address 对象
        start_address = ipaddress.ip_address(start_str)
        end_address = ipaddress.ip_address(end_str)
        # 将要判断的地址字符串转换为 IPv6Address 对象
        ip_address = ipaddress.ip_address(ip_str)
        # 判断地址是否在范围内
        return start_address <= ip_address <= end_address

        # 示例用法

    @staticmethod
    def mac_generator(number: int, max: int = 10, **kwargs):
        """
        生成MAC地址
        """
        a: int = kwargs.get("a", 255)
        b: int = kwargs.get("b", 255)
        c: int = kwargs.get("c", 255)
        d: int = kwargs.get("d", 1)
        e: int = kwargs.get("e", 1)
        f: int = kwargs.get("f", 1)
        a_range: int = 256
        b_range: int = 256
        c_range: int = 256
        d_range: int = 256
        e_range: int = 256
        f_range: int = 256
        count = 0
        mac_list = []
        for _ in range(1, max+1):
            mac_list.append(f"{a:02x}:{b:02x}:{c:02x}:{d:02x}:{e:02x}:{f:02x}")
            count += 1
            f += 1
            if f == f_range:
                f = 1
                e = e+1
                if e == e_range:
                    e = 1
                    d = d+1
                    if d == d_range:
                        d = 1
                        c = c+1
                        if c == c_range:
                            c = 1
                            b = b+1
                            if b == b_range:
                                b = 1
                                a = a+1
                                if a == a_range:
                                    a = 1
            if len(mac_list) % number == 0:
                yield mac_list
                mac_list = []
            if max == count and len(mac_list) != 0:
                yield mac_list



import hashlib
import ipaddress
import random
import secrets
import shutil
import string
import uuid
import warnings
from typing import Literal

# 设置警告过滤器
warnings.filterwarnings("always")


class Random:
    """
    随机生成一些测试数据
    """
    @staticmethod
    def file(data: str | None = None) -> tuple:
        """
        生成随机文件并返回文件路径和文件的 MD5 哈希值。

        该函数用于生成一个随机文件，并返回该文件的路径和MD5哈希值。
        可以通过传入参数指定文件内容，默认为None。函数首先生成一个随机字符串作为
        临时文件的名称。然后，根据传入的内容将文件写入临时文件中。
        最后，使用hashlib库计算临时文件的MD5哈希值，并将临时文件移动到指定路径上。
        函数返回一个包含文件路径和MD5哈希值的元组。

        Args:
            data (str | None, optional): 文件内容，默认为 None。

        Returns:
            tuple: 包含文件路径和文件的 MD5 哈希值的元组。

        Example:
            ```python
            result = random_file(data='Hello, World!')
            print(result)  # 输出: ("./file", "a1b2c3d4e5f6...")
            ```
        """

        rand_str: list[str] = random.sample(
            string.ascii_letters + string.digits, 20)
        random_value: str = ''.join(rand_str) + '\n' + str(uuid.uuid4())

        temp_name = f'.{Random.random_hash()}'
        with open(temp_name, 'w') as w:
            if data is not None:
                w.write(f"{data}\n{random_value}")
            else:
                w.write(f"{random_value}")
        with open(temp_name, 'rb') as r:
            md5: str = hashlib.md5(r.read()).hexdigest()
        temp_path: str = f'./{md5}'
        shutil.move(temp_name, temp_path)
        return temp_path, md5

    @staticmethod
    def str(length: int = 1, is_zh: bool = False, mark: bool = False, is_int: bool = False, zh_code_is_unicode=False) -> str:
        """
        生成指定长度的随机大小写字母、数字组成的字符串。

        Args:
            length (int): 字符串长度。越短重复的概率越高。
            is_zh (bool, optional): 是否包含中文字符，默认为 False。
            mark (bool, optional): 默认生成的字符串是否包含标点符号，默认为 False。
            is_int (bool, optional): 是否生成随机整数字符串，默认为 False。
            zh_code_is_unicode (bool, optional): 使用Unicode编码生成中文字符。默认使用GBK编码生成更常见的中文字符。

        Returns:
            str: 生成的随机字符串。

        Examples:
        >>> print(Random.str(1))   

        """

        def Unicode() -> str:
            result: Literal[''] = ""
            for _ in range(length):
                char = chr(random.randint(0x4E00, 0x9FA5))
                result += char
            return result

        def GBK2312() -> str:
            result: Literal[''] = ""
            for _ in range(length):
                head = random.randint(0xb0, 0xf7)
                body = random.randint(0xa1, 0xfe)
                val = f'{head:x} {body:x}'
                str = bytes.fromhex(val).decode('gb2312')
                result += str
            return result

        if is_zh:
            if zh_code_is_unicode:
                return Unicode()
            else:
                return GBK2312()

        if is_int:
            return ''.join(str(secrets.randbelow(10)) for _ in range(length))

        # 选择字符集
        characters = string.ascii_letters + string.digits

        if mark:
            characters += string.punctuation

        sys_random = random.SystemRandom()
        return "".join(sys_random.choice(string.ascii_letters + string.digits) for _ in range(length))

    @staticmethod
    def hash(len: int | None = None) -> str:
        """
        生成一个随机的MD5哈希字符串。

        Args:
        - len: int | None - 生成的哈希字符串的长度。如果为None，则返回完整的MD5哈希字符串（32个字符）。

        Returns:
        - str: 生成的MD5哈希字符串。根据`len`参数可能被截断。
        """

        letters = string.printable
        rand = ''.join(random.sample(letters, 10))
        hash = hashlib.md5(rand.encode()).hexdigest()
        return hash[:len]

    @staticmethod
    def hash_uuid4(len: int | None = None) -> str:
        """
        生成一个基于UUID4的随机哈希字符串。

        Args:
        - len: 需要返回的哈希字符串的长度。如果为None，则返回整个哈希字符串。

        Returns:
        - str: 根据指定长度生成的哈希字符串。
        """

        u4 = uuid.uuid4()
        hash = hashlib.md5(u4.bytes).hexdigest()
        return hash[:len]

    @staticmethod
    def ipv4():
        """
        生成一个随机的IPv4地址。

        Returns:
        - IPv4Address: 一个随机生成的IPv4地址。
        """
        warnings.warn(f"`{Random.ipv4.__name__}`已弃用即将删除，改用`Random.ip`",
                      DeprecationWarning, stacklevel=2)
        return ipaddress.IPv4Address(random.randint(0, 2**32-1)).__str__()

    @staticmethod
    def ipv6():
        """
        生成一个随机的IPv6地址。

        Returns:
        - IPv6Address: 返回一个随机生成的IPv6地址。
        """

        warnings.warn(f"`{Random.ipv6.__name__}`已弃用即将删除，改用`Random.ip`",
                      DeprecationWarning, stacklevel=2)
        return ipaddress.IPv6Address(random.randint(0, 2**128-1)).__str__()

    @staticmethod
    def ip(is_ipv6: bool = False) -> str:
        """
        生成一个随机的IP地址。
        """
        if is_ipv6:
            return ipaddress.IPv6Address(random.randint(0, 2**128-1)).__str__()
        else:
            return ipaddress.IPv4Address(random.randint(0, 2**32-1)).__str__()

    @staticmethod
    def ip_suffix_is_0(is_ipv6: bool = False, suffix: str = 0) -> str:
        """
        生成一个随机后缀为`0`|`suffix`的IP地址。
        """
        if is_ipv6:
            return ":".join(
                [f"{random.randint(0, 65535):x}" for _ in range(4)]) + f"::{suffix}"
        else:
            return ".".join(
                [f"{random.randint(0, 255)}" for _ in range(3)]) + f".{suffix}"

    @staticmethod
    def ip_range(is_ipv6: bool = False):
        """
        生成一个IP地址范围，可以是IPv4或IPv6地址格式。

        Args:
        - is_ipv6: bool 值，用于指定生成IPv4还是IPv6地址范围。默认为False，表示生成IPv4地址范围。

        Returns:
        - 返回一个字符串，表示IP地址范围。如果is_ipv6为True，则返回IPv6地址范围，否则返回IPv4地址范围。
        """
        if is_ipv6:
            ip = ":".join(
                [f"{random.randint(0, 65535):x}" for _ in range(4)])
            return f"{ip}::{random.randint(1, 10000):x}-{ip}::{random.randint(20000, 65500):x}"
        else:
            ip = ".".join(
                [f"{random.randint(0, 255)}" for _ in range(3)])
            return f"{ip}.{random.randint(1, 100)}-{ip}.{random.randint(110, 254)}"

    @staticmethod
    def ip_netmask(is_ipv6: bool = False, netmask: int | None = None):
        """
        生成一个随机的IP子网掩码格式。
        """
        if is_ipv6:
            netmask = netmask if netmask is not None else random.randint(
                64, 128)
            return f"{Random.ip_suffix_is_0(True)}/{netmask}"
        else:
            netmask = netmask if netmask is not None else random.randint(8, 32)
            return f"{Random.ip_suffix_is_0()}/{netmask}"

    @staticmethod
    @property
    def mac():
        """
        生成一个随机的MAC。
        """
        mac_address = ':'.join(['{:02x}'.format(
            (uuid.getnode() >> elements) & 0xff) for elements in range(0, 2*6, 2)])
        return mac_address
